from CvPythonExtensions import *
import CvUtil

gc = CyGlobalContext()


#################################################################################
#										#
#			       +++> zPromotions <+++				#
#										#
#^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^#
#-------------------------------------------------------------------------------#
#			GNU General Public License				#
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -#
# This file is part of zPromotions mod.						#
#										#
# zPromotions mod is free software; you can redistribute			#
# it and/or modify it under the terms of the GNU General Public			#
# License as published by the Free Software Foundation; either			#
# version 2 of the License, or (at your option) any later version.		#
#										#
# zPromotions mod is distributed in the hope that it will			#
# be useful, but WITHOUT ANY WARRANTY; without even the implied			#
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.		#
# See the GNU General Public License for more details.				#
#										#
# You should have received a copy of the GNU General Public License		#
# along with zPromotions mod; if not, write to the Free				#
# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA		#
# 02110-1301 USA								#
#										#
#-------------------------------------------------------------------------------#
#										#
#			OVERVIEW						#
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -#
#	To start with all function names starting with "get" have return and all#
# functions that start with "set" return nothing. 				#
#	There are 2 classes, they are "promotion" & "receive". Promotion is used#
# to hold all data about each promotion that has it's own enslavement		#
# properties. Receive interfaces with promotion.				#
#										#
#################################################################################




#################################################################################
#										#
#			    +++> promotion Class <+++				#
#										#
#################################################################################
#			OVERVIEW						#
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -#
#	The promotion class is used to hold individual data about when and	#
# where to give a promotion.							#
#-------------------------------------------------------------------------------#
#			Here is the list of functions and their specs.		#
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -#
#	VOID setPromoType(PromoType type)					#
#		sets the promo type						#
#			Note: PromoType is a STRING not an INT			#
#										#
#	STRING getPromoType()							#
#		returns the promo type						#
#										#
#	VOID setPromoEvent(PromoEvent event)					#
#		sets the promo event can be:					#
#			1> "PROMO_COMBAT"					#
#			2> "PROMO_PLOT"						#
#			3> "PROMO_COMBAT_PLOT"					#
#			4> "PROMO_LOSER_COMBAT"					#
#			5> "PROMO_HOMELAND"					#
#			6> "PROMO_CITY_BUILDING"				#
#			7> "PROMO_BUILT"					#
#			8> "PROMO_NUETRAL"					#
#			9> "PROMO_BARBARIAN"					#
#			10> "PROMO_VASSAL"					#
#			11> "PROMO_MASTER"					#
#			12> "PROMO_FRIENDLY"					#
#			13> "PROMO_ENEMY"					#
#			14> "PROMO_HOSTILE"					#
#			15> "PROMO_AIR_CARRIER"					#
#			Note: PromoEvent is a STRING not an INT			#
#										#
#	STRING getPromoEvent()							#
#		returns the promo event						#
#										#
#	VOID setPromoChance(INT chance)						#
#		sets the promo chance						#
#			Note: chance should not be above 100 or below 0		#
#										#
#	INT getPromoChance()							#
#		returns the promo chance					#
#										#
#	VOID addOrData(STRING type)						#
#		adds type to orData						#
#			type can be:						#
#			1> Any valid terrain type				#
#			2> Any valid feature type				#
#			3> Any valid improvement type				#
#			4> Any valid bonus type					#
#			5> Any valid building type				#
#			Note: type is a STRING not an INT			#
#			there is no defined getFunction for addPlotData		#
#										#
#	VOID addAndData(STRING type)						#
#		adds type to andData						#
#			type can be:						#
#			1> Any valid promotion					#
#			2> Any valid technology					#
#			Note: type is a STRING not an INT			#
#			there is no defined getFunction for addPlotData		#
#										#
#	STRING doGivePromo(data1, data2, data3)					#
#		calls the correct promotion giving function			#
#			data1 should be a unit or None				#
#			data2 should be a unit or None				#
#			data3 should be a plot or None				#
#										#
#	BOOL evalANDPrerequisites(UnitInstance pUnit)				#
#		returns True if pUnits team has the correct techs and pUnit has	#
#		the correct promotions otherwise returns False			#
#										#
#	BOOL evalUnitClass(UnitInstance pUnit)					#
#		returns true if any the unit class is correct otherwise it	#
#		returns	false							#
#										#
#	PROMO_COMBAT, PROMO_PLOT, PROMO_COMBAT_PLOT, PROMO_LOSER, PROMO_HOMELAND#
#	, PROMO_CITY_BUILDING, and PROMO_BUILT should not be called except by	#
#	doGivePromo								#
#										#
#################################################################################

class promotion:
	promoType = None
	promoChance = 0
	promoTech = None
	promoObsolete = None
	orData = list()
	andData = list()
	unitData = list()
	promoEvent = None
	promoKeep = False
	promoReceive = True
	id = -1


	def setPromoType(self, type):								#sets the promo type
		self.promoType = type

	def getPromoType(self):									#returns the promo type
		return self.promoType

	def setPromoEvent(self, event):								#sets the promo type
		self.promoEvent = event

	def getPromoEvent(self):								#returns the promo type
		return self.promoEvent

	def setPromoChance(self, chance):							#sets the promo chance
		if chance > 100:
			self.promoChance = 100
		elif chance < 0:
			self.promoChance = 0
		else:
			self.promoChance = chance

	def getPromoChance(self):								#returns the promo chance
		return self.promoChance

	def addOrData(self, type):								# Adds the required data to orData
		for i in range(len(self.orData)):
			if self.orData[i][0] == type:
				return

		self.orData.append(type)

	def addAndData(self, type):								# Adds the required data to andData
		for i in range(len(self.andData)):
			if self.andData[i][0] == type:
				return

		self.andData.append(type)

	def PROMO_COMBAT(self, pWinner, pLoser, pPlot):						# Called By doGivePromo
		if pLoser == None or pWinner == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_COMBAT') + 1
		if rand <= self.getPromoChance():
			pWinner.setHasPromotion(iPromo, self.promoReceive)
			if self.promoReceive == True:
				return gc.getPromotionInfo(iPromo).getDescription()
		return ""

	def PROMO_LOSER_COMBAT(self, pLoser, pWinner, pPlot):						# Called By doGivePromo This is all little backwards so that it checks the loser for the promotion
		if pLoser == None or pWinner == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_LOSER_COMBAT') + 1
		if rand <= self.getPromoChance():
			pWinner.setHasPromotion(iPromo, self.promoReceive)
			if self.promoReceive == True:
				return gc.getPromotionInfo(iPromo).getDescription()
		return ""

	def PROMO_PLOT(self, pUnit, junk, pPlot):						# Called By doGivePromo
		if pUnit == None or pPlot == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_COMBAT') + 1
		if len(self.orData) == 0:
			if rand <= self.getPromoChance():
				pUnit.setHasPromotion(iPromo, self.promoReceive)
				return ""
		else:
			for iPlotData in range(len(self.orData)):
				#########################################################################
				#									#
				# A ton of logic code							#
				#	 ****>>> ALL LOGIC SHOULD BE DONE LIKE BELOW <<<****		#
				#									#
				#########################################################################
				iTerrain = self.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), self.orData[iPlotData])
				terrain = self.check(iTerrain, pPlot.getTerrainType() == iTerrain)
				iFeature = self.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), self.orData[iPlotData])
				feature = self.check(iFeature, pPlot.getFeatureType() == iFeature)
				iImprovement = self.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), self.orData[iPlotData])
				improvement = self.check(iImprovement, pPlot.getImprovementType() == iImprovement)
				iBonus = self.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), self.orData[iPlotData])
				bonus = self.check(iBonus, pPlot.getBonusType(-1) == iBonus)

				if (terrain or feature or improvement or bonus) and rand <= self.getPromoChance():
					pUnit.setHasPromotion(iPromo, self.promoReceive)
					return ""
				elif self.promoKeep == False and self.promoReceive == True:
					pUnit.setHasPromotion(iPromo, False)
					return ""
		return ""

	def PROMO_COMBAT_PLOT(self, pUnit, pLoser, junk):					# Called By doGivePromo
		if pUnit == None or pLoser == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		pPlot = pLoser.plot()
		rand = CyGame().getSorenRandNum(100, 'PROMO_COMBAT') + 1
		if len(self.orData) == 0:
			if rand <= self.getPromoChance():
				pUnit.setHasPromotion(iPromo, self.promoReceive)
				if self.promoReceive == True:
					return gc.getPromotionInfo(iPromo).getDescription()
		else:
			for iPlotData in range(len(self.orData)):
				#########################################################################
				#									#
				# A ton of logic code							#
				#	 ****>>> ALL LOGIC SHOULD BE DONE LIKE BELOW <<<****		#
				#									#
				#########################################################################
				iTerrain = self.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), self.orData[iPlotData])
				terrain = self.check(iTerrain, pPlot.getTerrainType() == iTerrain)
				iFeature = self.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), self.orData[iPlotData])
				feature = self.check(iFeature, pPlot.getFeatureType() == iFeature)
				iImprovement = self.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), self.orData[iPlotData])
				improvement = self.check(iImprovement, pPlot.getImprovementType() == iImprovement)
				iBonus = self.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), self.orData[iPlotData])
				bonus = self.check(iBonus, pPlot.getBonusType(-1) == iBonus)

				if (terrain or feature or improvement or bonus) and rand <= self.getPromoChance():
					pUnit.setHasPromotion(iPromo, self.promoReceive)
					if self.promoReceive == True:
						return gc.getPromotionInfo(iPromo).getDescription()
		return ""

	def PROMO_HOMELAND(self, pUnit, junk, pPlot):						# Called By doGivePromo
		if pUnit == None or pPlot == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_COMBAT') + 1
		if pPlot.getOwner() == pUnit.getOwner() and rand <= self.getPromoChance():
			pUnit.setHasPromotion(iPromo, self.promoReceive)
			return ""
		elif not self.promoKeep:
			pUnit.setHasPromotion(iPromo, False)
			return ""
		return ""

	def PROMO_NEUTRAL(self, pUnit, junk, pPlot):						# Called By doGivePromo
		if pUnit == None or pPlot == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_NEUTRAL') + 1
		if pPlot.getOwner() == -1 and rand <= self.getPromoChance():
			pUnit.setHasPromotion(iPromo, self.promoReceive)
			return ""
		elif not self.promoKeep:
			pUnit.setHasPromotion(iPromo, False)
			return ""
		return ""

	def PROMO_BARBARIAN(self, pUnit, junk, pPlot):						# Called By doGivePromo
		if pUnit == None or pPlot == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_NEUTRAL') + 1
		if pPlot.getOwner() == gc.getBARBARIAN_PLAYER() and rand <= self.getPromoChance():
			pUnit.setHasPromotion(iPromo, self.promoReceive)
			return ""
		elif not self.promoKeep:
			pUnit.setHasPromotion(iPromo, False)
			return ""
		return ""

	def PROMO_VASSAL(self, pUnit, junk, pPlot):						# Called By doGivePromo
		if pUnit == None or pPlot == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_NEUTRAL') + 1
		if gc.getTeam(pPlot.getTeam()).isVassal(pUnit.getTeam()) and rand <= self.getPromoChance():
			pUnit.setHasPromotion(iPromo, self.promoReceive)
			return ""
		elif not self.promoKeep:
			pUnit.setHasPromotion(iPromo, False)
			return ""
		return ""

	def PROMO_MASTER(self, pUnit, junk, pPlot):						# Called By doGivePromo
		if pUnit == None or pPlot == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_NEUTRAL') + 1
		if gc.getTeam(pUnit.getTeam()).isVassal(pPlot.getTeam()) and rand <= self.getPromoChance():
			pUnit.setHasPromotion(iPromo, self.promoReceive)
			return ""
		elif not self.promoKeep:
			pUnit.setHasPromotion(iPromo, False)
			return ""
		return ""

	def PROMO_FRIENDLY(self, pUnit, junk, pPlot):						# Called By doGivePromo
		if pUnit == None or pPlot == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_NEUTRAL') + 1
		if gc.getTeam(pUnit.getTeam()).isOpenBorders(pPlot.getTeam()) and rand <= self.getPromoChance():
			pUnit.setHasPromotion(iPromo, self.promoReceive)
			return ""
		elif not self.promoKeep:
			pUnit.setHasPromotion(iPromo, False)
			return ""
		return ""

	def PROMO_ENEMY(self, pUnit, junk, pPlot):						# Called By doGivePromo
		if pUnit == None or pPlot == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_NEUTRAL') + 1
		if gc.getTeam(pUnit.getTeam()).isAtWar(pPlot.getTeam()) and rand <= self.getPromoChance():
			pUnit.setHasPromotion(iPromo, self.promoReceive)
			return ""
		elif not self.promoKeep:
			pUnit.setHasPromotion(iPromo, False)
			return ""
		return ""

	def PROMO_HOSTILE(self, pUnit, junk, pPlot):						# Called By doGivePromo
		if pUnit == None or pPlot == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_NEUTRAL') + 1
		if not gc.getTeam(pUnit.getTeam()).isAtWar(pPlot.getTeam()) and not gc.getTeam(pUnit.getTeam()).isOpenBorders(pPlot.getTeam()) and not gc.getTeam(pUnit.getTeam()).isVassal(pPlot.getTeam()) and not gc.getTeam(pPlot.getTeam()).isVassal(pUnit.getTeam()) and rand <= self.getPromoChance():
			pUnit.setHasPromotion(iPromo, self.promoReceive)
			return ""
		elif not self.promoKeep:
			pUnit.setHasPromotion(iPromo, False)
			return ""
		return ""

	def PROMO_AIR_CARRIER(self, pUnit, junk, junk2):						# Called By doGivePromo
		if pUnit == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_NEUTRAL') + 1

		numCargo = pUnit.getCargo()
		cargoType = pUnit.specialCargo()
		if numCargo > 0 and cargoType == gc.getInfoTypeForString("SPECIALUNIT_FIGHTER") and rand <= self.getPromoChance():
			pUnit.setHasPromotion(iPromo, self.promoReceive)
			return ""
		elif not self.promoKeep:
			pUnit.setHasPromotion(iPromo, False)
			return ""
		return ""

	def PROMO_CITY_BUILDING(self, pUnit, junk, pPlot):					# Called By doGivePromo
		if pUnit == None or pPlot == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_COMBAT') + 1
		if pPlot.getOwner() == pUnit.getOwner() and pPlot.isCity():
			if len(self.orData) == 0:
				if rand <= self.getPromoChance():
					pUnit.setHasPromotion(iPromo, self.promoReceive)
					return ""
			else:
				for iPlotData in range(len(self.orData)):
					iBuilding = self.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), self.orData[iPlotData])
					building = self.check(iBuilding, pPlot.getPlotCity().isHasRealBuilding(iBuilding))
					if building and rand <= self.getPromoChance():
						pUnit.setHasPromotion(iPromo, self.promoReceive)
						return ""
		elif not self.promoKeep:
			pUnit.setHasPromotion(iPromo, False)
			return ""
		return ""

	def PROMO_BUILT(self, pUnit, junk, pPlot):					# Called By doGivePromo
		if pUnit == None or pPlot == None:
			return ""
		iPromo = gc.getInfoTypeForString(self.getPromoType())
		rand = CyGame().getSorenRandNum(100, 'PROMO_COMBAT') + 1
		if len(self.orData) == 0:
			if rand <= self.getPromoChance():
				pUnit.setHasPromotion(iPromo, self.promoReceive)
				return ""
		else:	
			for iPlotData in range(len(self.orData)):
				#########################################################################
				#									#
				# A ton of logic code							#
				#	 ****>>> ALL LOGIC SHOULD BE DONE LIKE BELOW <<<****		#
				#									#
				#########################################################################
				iBuilding = self.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), self.orData[iPlotData])
				building = self.check(iBuilding, pPlot.getPlotCity().isHasRealBuilding(iBuilding))
				iTerrain = self.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), self.orData[iPlotData])
				terrain = self.check(iTerrain, pPlot.getTerrainType() == iTerrain)
				iFeature = self.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), self.orData[iPlotData])
				feature = self.check(iFeature, pPlot.getFeatureType() == iFeature)
				iImprovement = self.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), self.orData[iPlotData])
				improvement = self.check(iImprovement, pPlot.getImprovementType() == iImprovement)
				iBonus = self.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), self.orData[iPlotData])
				bonus = self.check(iBonus, pPlot.getBonusType(-1) == iBonus)

				if ((building and pPlot.getOwner() == pUnit.getOwner()) or terrain or feature or improvement or bonus) and rand <= self.getPromoChance():
					pUnit.setHasPromotion(iPromo, self.promoReceive)
					return ""
				elif not self.promoKeep:
					pUnit.setHasPromotion(iPromo, False)
					return ""
		return ""

	def doGivePromo(self, data1, data2, data3):						# Calls The Correct Function For This Promotion
		if data1.isHasPromotion(gc.getInfoTypeForString(self.getPromoType())) == False and self.promoReceive == False:
			return ""
		if data1.isHasPromotion(gc.getInfoTypeForString(self.getPromoType())) == True and self.promoReceive == True:
			return ""
		if self.evalANDPrerequisites(data1) == False or self.evalUnitClass(data1) == False:		#If the unit class is wrong or the winner doesn't have the right tech exit
			return ""
		#If you add your own event your event function should check to make sure that the data it received is correct and if the is not correct return an empty string ("")
		#You will need to add your event function above and put an event key in the below dict and the name of your event function
		#Then you will need to add the event key to the dict located in received classes promos dict and set it to be a list
		#Next you will have to add your event key to each call made from the custom event manager made to the doReceive function, it is best to only set it on the calls that you want to work with (this also makes the code run faster).
		self.promoEventMask = {	"PROMO_COMBAT"		: self.PROMO_COMBAT,
					"PROMO_PLOT" 		: self.PROMO_PLOT,
					"PROMO_COMBAT_PLOT" 	: self.PROMO_COMBAT_PLOT,
					"PROMO_HOMELAND"	: self.PROMO_HOMELAND,
					"PROMO_CITY_BUILDING"	: self.PROMO_CITY_BUILDING,
					"PROMO_BUILT"		: self.PROMO_BUILT,
					"PROMO_NEUTRAL"		: self.PROMO_NEUTRAL,
					"PROMO_LOSER_COMBAT"	: self.PROMO_LOSER_COMBAT,
					"PROMO_BARBARIAN"	: self.PROMO_BARBARIAN,
					"PROMO_VASSAL"		: self.PROMO_VASSAL,
					"PROMO_MASTER"		: self.PROMO_MASTER,
					"PROMO_FRIENDLY"	: self.PROMO_FRIENDLY,
					"PROMO_ENEMY"		: self.PROMO_ENEMY,
					"PROMO_HOSTILE"		: self.PROMO_HOSTILE,
					"PROMO_AIR_CARRIER"	: self.PROMO_AIR_CARRIER}

		return self.promoEventMask[self.getPromoEvent()](data1, data2, data3)

	def evalANDPrerequisites(self, pUnit):
		for iPlotData in range(len(self.andData)):
			iTech = self.findInfoTypeNum(gc.getTechInfo, gc.getNumTechInfos(), self.andData[iPlotData])
			iPromo = self.findInfoTypeNum(gc.getPromotionInfo, gc.getNumPromotionInfos(), self.andData[iPlotData])
			if gc.getTeam(gc.getPlayer(pUnit.getOwner()).getTeam()).isHasTech(iTech) and iTech >= 0:
				pass
			elif pUnit.isHasPromotion(iPromo) and iPromo >= 0:
				pass
			else:
				return False
		return True

	def evalUnitClass(self, pUnit):
		if len(self.unitData) == 0:		#If orData has nothing in it return True, the user wants the unit to receive a promo based on promotions and techs
			return True
		ret = True
		for iUnitData in range(len(self.unitData)):
			if pUnit.getUnitClassType() == self.findInfoTypeNum(gc.getUnitClassInfo, gc.getNumUnitClassInfos(), self.unitData[iUnitData]):
				return True
			elif pUnit.getUnitClassType() != self.findInfoTypeNum(gc.getUnitClassInfo, gc.getNumUnitClassInfos(), self.unitData[iUnitData]):
				ret = False
		return ret

	def findInfoTypeNum(self, infoGetter = None, numInfos = None, typeStr = None):
		if infoGetter == None or numInfos == None or typeStr == None:
			return -1
		for i in range(numInfos):
			if infoGetter(i).getType() == typeStr:
				return i
		return -1

	def check(self, intToPass, result):		#Takes an int makes sure that it is greater than -1 if it is less than 0 it returns false otherwise it returns the value of result
		if intToPass < 0:
			return False
		return result

#################################################################################
#										#
#			    +++> receive Class <+++				#
#										#
#################################################################################
#			OVERVIEW						#
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -#
#	The receive class is used to interface and hold data in class promotion.#
#-------------------------------------------------------------------------------#
#			Here is the list of functions and their specs.		#
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -#
#	PROMOTION addPromoType(promoType, eventType, promoChance, data,		#
# receive, keep)								#
#		adds a new promotion to the promos list then returns it		#
#			Note: POMOTION is my promotion class			#
#										#
#	VOID doReceive(data1, data2, data3, x, y)				#
#		returns the promo type						#
#		calls the correct promotion giving function			#
#			data1 should be a unit or None				#
#			data2 should be a unit or None				#
#			data3 should be a plot or None				#
#			x should be the x coord of the plot where the event	#
#				happend						#
#			y should be the y coord of the plot where the event	#
#				happend						#
#										#
#################################################################################
class receive:
	promos = {	"PROMO_COMBAT"		: list(),
			"PROMO_PLOT" 		: list(),
			"PROMO_COMBAT_PLOT" 	: list(),
			"PROMO_HOMELAND"	: list(),
			"PROMO_CITY_BUILDING"	: list(),
			"PROMO_BUILT"		: list(),
			"PROMO_NEUTRAL"		: list(),
			"PROMO_LOSER_COMBAT"	: list(),
			"PROMO_BARBARIAN"	: list(),
			"PROMO_VASSAL"		: list(),
			"PROMO_MASTER"		: list(),
			"PROMO_FRIENDLY"	: list(),
			"PROMO_ENEMY"		: list(),
			"PROMO_HOSTILE"		: list(),
			"PROMO_AIR_CARRIER"	: list()}

	promoButton = 'Python/Valence.dds'							# Holds the location of the image to display

	def addPromoType(self, promoType, eventType, promoChance, andData, orData, unitData, receive, keep, id):		#creates a new unit object and adds it to the units list

		if gc.getInfoTypeForString(promoType) < 0:
			return

		for iPromos in range(len(self.promos[eventType])):
			if self.promos[eventType][iPromos].id == id:
				self.promos[eventType][iPromos].setPromoType(promoType)
				self.promos[eventType][iPromos].setPromoChance(promoChance)
				self.promos[eventType][iPromos].setPromoEvent(eventType)
				if orData != None:
					self.promos[eventType][iPromos].orData = orData
				if andData != None:
					self.promos[eventType][iPromos].andData = andData
				if unitData != None:
					self.promos[eventType][iPromos].unitData = unitData
				self.promos[eventType][iPromos].promoKeep = keep
				self.promos[eventType][iPromos].promoReceive = receive
				return self.promos[eventType][iPromos]
		class newPromo(promotion):
			def __init__(self):
				self.setPromoType(promoType)
				self.setPromoChance(promoChance)
				self.setPromoEvent(eventType)
				if orData != None:
					self.orData = orData
				if andData != None:
					self.andData = andData
				if unitData != None:
					self.unitData = unitData
				self.promoKeep = keep
				self.promoReceive = receive
				self.id = id
		
		self.promos[eventType].append(newPromo())
		return self.promos[eventType][len(self.promos[eventType]) - 1]

	def doReceive(self, data1, data2, data3, x, y, events):						# Goes through the promos list and calls that promos doGivePromo function
		print ">>>Starting Receive :: Z9 Debug For zPromotions"
		receivedPromos = ""
		num = 0
		for i in range(len(events)):
			print ">>>Searching %s For Promotions :: Z9 Debug For zPromotions" %(events[i])
			for j in range(len(self.promos[events[i]])):
				print ">>>Preparing To Give Promotion :: Z9 Debug For zPromotions"
				newPromo = self.promos[events[i]][j].doGivePromo(data1, data2, data3)
				print ">>>Gave Promotion -->%s<-- (if noting between arrows no promotion alert) :: Z9 Debug For zPromotions" %(newPromo)
				if receivedPromos == "" and newPromo != "":
					receivedPromos = newPromo
					num += 1
				elif newPromo != "":
					receivedPromos += ", " + newPromo
					num += 1
		if num == 1:
			CyInterface().addMessage(data1.getOwner(), False, 25, 'Your %s Has Recived a Promotion From Combat (%s)!' %(gc.getUnitInfo(data1.getUnitType()).getDescription(), receivedPromos), 'AS2D_FEAT_ACCOMPLISHED', 1, self.promoButton, 8, x, y, True, True)
		elif num > 1:
			CyInterface().addMessage(data1.getOwner(), False, 25, 'Your %s Has Recived %d Promotions From Combat (%s)!' %(gc.getUnitInfo(data1.getUnitType()).getDescription(), num, receivedPromos), 'AS2D_FEAT_ACCOMPLISHED', 1, self.promoButton, 8, x, y, True, True)
